/***********************************************************************
**                                                                    **
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    **
***********************************************************************/
#define __PREPARE__
#include "prep_types.h"
#include "prepare.h"

FILE *global_defs;
FILE *struct_h;
FILE *struct_def;

char *input,*helpstr,*fc;
string struct_file,header_file,include_path,include_path1;
string function_name,definitions,orig_defs,sensitivity;

long_name svar,new_name;
string *all_names, mycommand;
int *all_types;
int i,j,k,l,num_vars=0;
int bp1,bp2,sb1,sb2,cp;
int type,pass,keyword,comment=0;
int get_int,box;
float get_flt;
unsigned int max_changes=0;
unsigned int num_changes=0;

int box_entries[MAX_DIMENSION];
int boxes=-1,boxes1=-1,boxes2=-1,boxes3=-1;
int box,box1,box2,box3;

void strtrim(char*);


int main(int argc, char *argv[])
  {
  void pass0(void);
  void pass1(void);

/* Uebergebene Argumente retten */
  sprintf(definitions,"%s",argv[1]);
  sprintf(struct_file,"%s",argv[2]);
  sprintf(header_file,"%s",argv[3]);
  sprintf(function_name,"%s",argv[4]);
  sprintf(include_path,"%s",argv[5]);
  sprintf(include_path1,"%s",argv[6]);
  sprintf(orig_defs,"%s",argv[7]);
  sprintf(sensitivity,"%s",argv[8]);


  all_names=(string *) malloc(total_vars*sizeof(string));
  all_types=(int *) malloc(total_vars*sizeof(int));

 
/* Files fuer Strukturdefinitionen oeffen
**         struc_c: hier werden die Strukuren vorbereitet
**         struct_h : Include file mit den globalen Deklarationen
*/

  input=(char *) malloc(2*_BUFSIZE_*sizeof(char));
  helpstr=(char *) malloc(_BUFSIZE_*sizeof(char));

  struct_def=fopen(struct_file,"wb");
  struct_h=fopen(header_file,"wb");
  fprintf(struct_def,auto_header);
  fprintf(struct_h,auto_header);

/* File mit den Modell setups oeffnen */

  global_defs=fopen(definitions,"rb");


/* Im ersten Durchgang wird:
**     1. Der File Header erzeugt und
**     2. die globalen Variablen deklariert
*/

  fprintf(struct_def,"#include \"stdinc.h\" \n");
  fprintf(struct_def,"#include \"constructs.h\" \n");
  fprintf(struct_def,"#include \"types.h\" \n");

  fprintf(struct_h,"void %s(void);\n",function_name);

  for (pass=0;pass<=2;pass++)
    {
    comment=0;
    type=0;
    while ( fgets(input,100,global_defs) != 0 )
      {
/*Pr�rozessor-"Mll"  entfernen */
      if (strpos(input,"#")>=0) sprintf(input,"%s","");

      cp=strpos(input,rem1);
      comment=cp-strpos(input,rem2);
      if (cp==0 && comment < 0) sprintf(input,"%s","");

      while (comment > 0)
        {
        if (fgets(input,100,global_defs) == 0 ) goto stop_label;
        if (strpos(input,rem2) >= 0)
          {
          comment=-1;
          if (fgets(input,100,global_defs) == 0 ) goto stop_label;
          }

        } /* ende von comment>=0 */
        keyword = check_keyword(&type,input);
      
      strtrim(input);

      if (keyword==0 && type !=0 && strlen(input)>3)
      {
      sb1=strpos(input,"{");
      sb2=strpos(input,"}");
      if (sb1<0 && sb2>0) 
      {
          printf("Error in %s in line containing %s. Missing or wrong delimiter.\nExiting!\n",orig_defs,input);
          exit(1);
      }
      if (sb1>0)
       {
        while(sb2<0)
         {
         fc=fgets(helpstr,100,global_defs);
         strcat(input,helpstr);
         sb2=strpos(input,"}");
         
         if (strpos(helpstr,"{")>0 || strpos(helpstr,"{")>0 || (sb2<0 && fc==0))
          {
               printf("Error in %s in line containing %s. Missing or wrong delimiter.\nExiting!\n",orig_defs,input);
              exit(1);
          }
         }
         strtrim(input);
         switch (pass)
            {
            case 0: pass0();
                    break;
            case 1: pass1();
                    break;
            case 2: 
            default: break;
            }
       }
       else
       {
        sprintf(helpstr,"%s",input);
        cp=strpos(helpstr,rem1);
        if (cp>0) sprintf(helpstr,"%s",strmid(helpstr,0,cp));

        while(strlen(helpstr)>0)
         {
         cp=strpos(helpstr,",");
         if (cp>0)
          {
          sb1=strlen(helpstr);
          sprintf(input,"%s",strmid(helpstr,0,cp));
          sprintf(helpstr,"%s",strmid(helpstr,cp+1,strlen(helpstr)-cp-1));
          } 
          else
          {
          sprintf(input,"%s",helpstr);
          sprintf(helpstr,"%s","");
          }
         if (strpos(input,";") < strpos(input,"{"))
         {
            printf("Error in %s in line containing %s. Missing or wrong delimiter.\nExiting!\n",orig_defs,input);
            exit(1);
         }

         switch (pass) 
            {
            case 0: pass0();
                    break;
            case 1: pass1();
                    break;
            case 2: 
            default: break;
            }
         }
        }

       } /* ende von if (keyword ... */
    }  /* ende von while fscan() != EOF */

    if (pass == 0)
        {
        rewind(global_defs);

        fprintf(struct_def,"extern int find_changes(char*,long_name*,real**,int*,int);\n");
        fprintf(struct_def,"void %s(string sensitivity) {\n",function_name);
        fprintf(struct_def,"long_name *change_name;\n");
        fprintf(struct_def,"string command;\n");
        fprintf(struct_def,"int *change_type;\n");
        fprintf(struct_def,"real **change_ptr;\n");
        fprintf(struct_def,"int found_change;\n");
        fprintf(struct_def,"change_name=(long_name*) malloc(%d*sizeof(long_name)); \n",
                            max_changes);
        fprintf(struct_def,"change_type=(int*) malloc(%d*sizeof(int)); \n",
                            max_changes);
        fprintf(struct_def,"change_ptr=(real**) malloc(%d*sizeof(real*)); \n",
                            max_changes);

        } 
        if (pass ==2) 
        {
/* Files schliessen nach dem zweiten Durchgang  */

        fclose(global_defs);



/* 
** Fuer jede Variable, ausser States, Globals und Derivs wird ein
** Sucheintrag erzeugt
*/

      for (j=0;j<num_vars;j++) 
        {

        type=all_types[j];
        switch (type)
          {
          case STATE:
          case GLOBAL: break;

          case INT_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=(real*) %s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,INT_PARAM);
                 num_changes++;
                 break;
          case REAL_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=%s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,REAL_PARAM);
                 num_changes++;
                 break;
    
          case REAL_CONST:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=&%s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,REAL_CONST);
                 fprintf(struct_def,"printf(\"Initializing float parameter: %s=%s from %s\\n\",(float) %s); \n",all_names[j],"%f",orig_defs,all_names[j]);
                 num_changes++;
                 break;

          case INT_CONST:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=(real*) &%s; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,INT_CONST);
                 fprintf(struct_def,"printf(\"Initializing integer parameter: %s=%s from %s\\n\",(int) %s); \n",all_names[j],"%d",orig_defs,all_names[j]);
                 num_changes++;
                 break;
          case DERIVES:break;

          case FEATURES:break;

          case _2D_INT_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=(real*) &%s[1][1]; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,_2D_INT_PARAM);
                 num_changes++;
                 break;
          case _2D_REAL_PARAM:fprintf(struct_def,"sprintf(change_name[%d],\"%s\",\"%s\"); ",num_changes,"%s",all_names[j]);
                 fprintf(struct_def,"change_ptr[%d]=&%s[1][1]; ",num_changes,all_names[j]);
                 fprintf(struct_def,"change_type[%d]=%d;\n",num_changes,_2D_REAL_PARAM);
                 num_changes++;
                 break;


          default:break;
          }

        }

/*         fprintf(struct_def,"#if defined (__verbose__)\n"); */
	fprintf(struct_def,"\nprintf(\"Initialized %d parameters from %s \\n\");\n",max_changes,orig_defs);
/*         fprintf(struct_def,"#endif\n"); */
        fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMCFILE_SUB);
fflush(struct_def);

/*new mycommand*/
        fprintf(struct_def,"sprintf(command,\"touch %s;\",sensitivity);\n","%s");
	
        fprintf(struct_def,"system(command);\n");
        fprintf(struct_def,"sprintf(command,\"cat %s %s > %s;\",sensitivity);\n",orig_defs,"%s",PARAMCFILE_SUB);
        fprintf(struct_def,"system(command);\n");
/*end mycommand*/

/*        fprintf(struct_def,"system(\"touch %s\");\n",sensitivity);
        fprintf(struct_def,"system(\"cat %s %s > %s\");\n",orig_defs,sensitivity,PARAMCFILE_SUB);*/

        fprintf(struct_def,"if (system(\"gcc -E -traditional -I%s -I%s %s  > %s\")!=0) std_error(\"Error in preprocessing the file %s. Check for unresolvable include statements.\");\n",include_path,include_path1,PARAMCFILE_SUB,PARAMTMPFILE_SUB,orig_defs);
        fprintf(struct_def,"\nprintf(\"Searching for changes of parameters from %s \\n\");\n",orig_defs);
        fprintf(struct_def,"system(\"mv -f %s %s\");\n",PARAMTMPFILE_SUB,PARAMCFILE_SUB);
        fprintf(struct_def,"cpp2c(\"%s\",\"%s\");\n",PARAMCFILE_SUB,PARAMTMPFILE_SUB);
        fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMCFILE_SUB);
        fprintf(struct_def,    "found_change=find_changes(\"%s\",change_name,change_ptr,change_type,%u);\n",PARAMTMPFILE_SUB,num_changes);
        fprintf(struct_def,"if (found_change==0) printf(\"No changes found.\\n\");");
        fprintf(struct_def,"fflush(stdout); \n");
	    fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMCFILE_SUB);

/*	fprintf(struct_def,"system(\"rm -f %s\");\n",PARAMTMPFILE);*/
	fprintf(struct_def,"return; \n");
        fprintf(struct_def,"} \n");

          fclose(struct_def);
          fclose(struct_h);
         }
    stop_label:;
    } /* ende von for (pass=0 ...  */
  free(input);
  free(helpstr);
return(0);
}


#include "pass0_s.c"
#include "pass1_s.c"

