/***********************************************************************
**                                                                    **
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    **
***********************************************************************/
#include <math.h>
#include "struct.h"
#include "constructs.h"
#include "methods.h"

#define MAXSTP 10000
#define TINY 1.0e-12
extern void control_main(int,real, real,real*, real*);
extern void rkqs(int,real*,real*,long,real*,real,real,real,real*,real*,real*);
extern void rkqc45(int,real*,real*,long,real*,real,real,real,real*,real*,real*);
extern void rkqc23(int,real*,real*,long,real*,real,real,real,real*,real*,real*);
extern real *yscal,*y,*dydx;

void odeint(real x1,real x2,real *h)
{
	int nstp;
        long i;
	real x,hnext=0.0,hdid, checkit;
        string error;

	x=x1;

        make_global_0D_states();
        make_local(DDD,CCC,KKK,y);
		
	for (nstp=1;nstp<=MAXSTP;nstp++) 
            {
/* 11 is the maximum for steps_of_int_to_go and shall be ok for all rk32, rk45 and rkck */
                control_main(11,x,*h,y,dydx);

		for (i=1;i<=nvar;i++) yscal[i]=fabs(y[i])+fabs(dydx[i]* *h)+TINY;

		checkit = (x+*h-x2)*(x+*h-x1);
		if (checkit > -TINY)
		  {
		   *h=x2-x;
                  }
                switch (abs(method))
                 {
                 case _RK23:rkqc23(method,y,dydx,nvar,&x,*h,acc,mindelt,yscal,&hdid,&hnext);
                        break;
                 case _RK45:rkqc45(method,y,dydx,nvar,&x,*h,acc,mindelt,yscal,&hdid,&hnext);
                        break;
                 case _RKCK:rkqs(method,y,dydx,nvar,&x,*h,acc,mindelt,yscal,&hdid,&hnext);
                        break;
                 }

		if ((x-x2)*(x2-x1) >= -TINY)
                   {
                      *h=fmin(maxdelt,hnext);
		      make_global(y,KKK,CCC,DDD);
                      make_local_0D_states();
		      return;
		   }
		if (fabs(hnext) <= mindelt)
                {
                    sprintf(error,"Step size too small in odeint. delt=%g",(double) hnext);
                    std_error(error);
                }
		*h=hnext;

	    }
	std_error("Too many steps in routine odeint");
}
#undef MAXSTP
#undef TINY
#undef NRANSI
