/***********************************************************************
**                                                                    **
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    **
***********************************************************************/
#include <math.h>
#include "struct.h"
#include "constructs.h"
#include "types.h"

#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4
#define FCOR 15
#define ZERO 0.0

extern void control_main(int,real, real,real*, real*);
extern void rk45(int,real*,real*,long,real,real,real*);
extern void shrunk(real,real,long,real,real,char*);
extern void warn_negative(real,long,real,char*);

extern real *dysav,*ysav,*ytemp;


void rkqc45(int meth,real *y,real *dydx,int n,real *x,real htry,real __acc,real __mindelt,real *yscal,real *hdid,real *hnext)
{
long i,shr_delt=0;
int nega,__accept_nega;
real xsav,hh,h,temp,errmax;
string error;

xsav=(*x);

for (i=1;i<=n;i++)
  {
  ysav[i]=y[i];
  dysav[i]=dydx[i];
  }


h=htry;
for (;;) 
  {
  hh=0.5*h;
  rk45(10,ysav,dysav,n,xsav,hh,ytemp);
  *x=xsav+hh;

  control_main(7,*x,hh,ytemp,dydx);
  rk45(6,ytemp,dydx,n,*x,hh,y);
                
  *x=xsav+h;

  if (fabs(h)<__mindelt) 
  {
     sprintf(error,"Step size too small in RKQC45. delt=%g",(double) h);
     std_error(error);
   }

  rk45(3,ysav,dysav,n,xsav,h,ytemp);

/* Unterdrueckung der Zeitschrittanpassung */
  if (__acc > 0.0)
    {
    }
    else
    {
     *hdid = h;
     *hnext= h;
     for (i=1;i<=n;i++) ytemp[i]=y[i]-ytemp[i];
     break;
    }

/* Zeitschrittanpassung */

  errmax=0.0;
  nega=0;
  __accept_nega=0;
  for (i=1;i<=n;i++) 
    {
#if defined (__accept_nega_with_zero_rate__)
                    if (y[i] < ZERO && ytemp[i] == y[i]) 
                   {
						__accept_nega=1;
#if defined (__verbose__)
					   warn_negative(*x,i,y[i],"rk45");
#endif
				    }
#endif			
    if (meth>0 && __accept_nega!=1 && (y[i] < ZERO || ytemp[i] < ZERO))
       {
        nega=1;
        shr_delt=i;
        break;
       }

    ytemp[i]=y[i]-ytemp[i];
    if (yscal[i]>0. || yscal[i]<0.) temp=fabs(ytemp[i]/yscal[i]); else temp=0.;

    if (errmax < temp) 
     {
     errmax=temp;
     shr_delt=i;
     }
    }
  errmax /= __acc;
  if (errmax <=1.0 && nega==0)
   {
   *hdid=h;
   *hnext=(errmax > ERRCON ? SAFETY*h*exp(PGROW*log(errmax)) : 4.0*h);
   break;
   }

   if (nega==1)
   {
     h/=2.;
   }
   else
   {
     h=SAFETY*h*exp(PSHRNK*log(errmax));
   }
   shrunk(*x,h,shr_delt,ysav[shr_delt],-(ysav[shr_delt]-y[shr_delt])/h,"rk45");
  }

for (i=1;i<=n;i++) y[i] += ytemp[i]/FCOR;

}
