/***********************************************************************
**                                                                    
** CEMoS Version 4.1.2 Copyright(C) 2023 Frank Hamberg, Cora Kohlmeier**
**                                                                    
**                                                                    
** Changes:                                                                   
**   05/08/01: check for multiple storage of variables
**                                                                 
***********************************************************************/
#include "stdinc.h"
#include "prepare.h"
#include "struct.h"
extern real* get_2D_store_ptr(int,char*);
void get_2D_stores(char *filename,int pre_dim,unsigned int *num_2D_stores,real ****ptr_store_2D_ptr,name **stores_2D)
{

   FILE *para_file;
   int cp=0,i=0,k=0;
   unsigned int i1,k1;
   int found=-1,counter=0;
   char *input,*list,*helpstr,*compstr;
   name *pre_stores;
   name *nsp;
   real ***pre_store_ptr;
   real **pointer=(real**) NULL,***ffp;

   para_file=fopen(filename,"rb");

   input=(char *) malloc(_BUFSIZE_*sizeof(char));
   helpstr=(char *) malloc(_BUFSIZE_*sizeof(char));
   compstr=(char *) malloc(_BUFSIZE_*sizeof(char));
   list=(char *) malloc(_BUFSIZE_*sizeof(char));
  
   pre_stores=(name *) malloc(pre_dim*sizeof(name));
   pre_store_ptr=(real ***) malloc(pre_dim*sizeof(real**));
   if (!pre_stores) std_error("Internal error: allocation failure for 2D_pre_stores");
   if (!pre_store_ptr) std_error("Internal error: allocation failure for 2D_pre_store_ptr");

/* store-Block suchen */
   while (found < 0)
    {
    if (fgets(helpstr,100,para_file) == 0) goto stop_label;
    found=strpos(helpstr,_store_2D_);
    }

#if defined (__verbose__)
    printf("2D Store variables:\n");
    fflush(stdout);
#endif
    while ( fgets(input,100,para_file) != 0 )
      {
      if (strpos(input,"#")>=0) sprintf(input,"%s","");

      while(strpos(input," ")==0) sprintf(input,"%s",strmid(input,1,(int) strlen(input)-1));

      if (strpos(input,change_id) ==0) break;
      if (strpos(input,_store_0D_)==0) break;
      if (strpos(input,_store_)==0) break;

      if (strpos(input,_keyword_) >=0) sprintf(input,"%s","");

      if (strlen(input)>0)
       {
#if defined (__verbose__)
        printf("%s",input);
        fflush(stdout);
#endif
        strtrim(input);
        sprintf(list,"%s",input);
        while(strlen(list)>0)
         {

         cp=strpos(list,";");
         if (cp>0)
          {
          sscanf(strmid(list,0,cp),"%s",helpstr);
          sprintf(list,"%s",strmid(list,cp+1,(int) strlen(list)-cp-1));
          }
          else
          {
          sscanf(list,"%s",helpstr);
          sprintf(list,"%s","");
          }

          found=0;
          i=(int) strlen(helpstr);
          for (k=1;k<=num_2D_states;k++)
           {
           sprintf(compstr,"%s",_2D_state_names[k]);
           if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
            {
            pointer= (real**) _2D_state_ptr[k];
            found=1;
            break;
            }
           }
           if (found==0)
            for (k=1;k<=num_2D_globals;k++)
            {
            sprintf(compstr,"%s",_2D_global_names[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
             {
            pointer= (real**) _2D_globals_ptr[k];
             found=1;
             break;
             }
            }
           if (found==0)
            for (k=1;k<=num_2D_float_derivs;k++)
            {
            sprintf(compstr,"%s",_2D_float_derivs[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
              {
              pointer= (real**) _2D_float_derivs_ptr[k];;
              found=1;
              break;
              }
             }
           if (found==0)
            for (k=1;k<=num_2D_glob_derivs;k++)
            {
            sprintf(compstr,"%s",_2D_glob_derivs[k]);
            if (strpos(compstr,helpstr) == 0 && (int)strlen(compstr)==i)
              {
              pointer= (real**) _2D_glob_derivs_ptr[k];;
              found=1;
              break;
              }
             }

	   if (pointer==(real**) 0 || found==0)
            {
            printf("Error in 2D-store command: No such Variable %s !!\n",helpstr);
            std_error("Check the the store-commands !!");
            }

           sprintf(pre_stores[*num_2D_stores],"%s",helpstr);
           pre_store_ptr[*num_2D_stores]=pointer;
           (*num_2D_stores) ++;

          }
       }
      }
    
    ffp=(real ***) malloc(*num_2D_stores * sizeof(real **));
    *stores_2D=(name *) malloc(*num_2D_stores * sizeof(name));
    nsp=*stores_2D;
/*  endgueltige Liste erstellen */
/* doppelte Eintraege aussortieren */

    counter = 0;
    for (i1=0;i1<*num_2D_stores;i1++)
    {
      found = 0;
      for (k1=0;k1<i1;k1++)
      {
         if ( i1 != k1 && pre_store_ptr[i1] == pre_store_ptr[k1]) 
         {
#if defined(__verbose__)
         printf("Warning: Removed multiple stored variable %s\n",pre_stores[i]);
#endif
         found = 1;
         }
      }
      if (found == 0) 
      {
      sprintf(nsp[counter],"%s",pre_stores[i1]);
      ffp[counter]=pre_store_ptr[i1];
      counter++;   
      }
    }
    *num_2D_stores=counter;
    *ptr_store_2D_ptr=ffp;
    
    stop_label:
    free(input);
    free(helpstr);
    free(compstr);
    free(list);
    free(pre_stores);
    free(pre_store_ptr);
    fclose(para_file);
}
