#!/bin/sh
# the next line restarts using wish\
exec wish "$0" "$@"
# CEMOS/CEMTK 5.0 Copyright(C) 2022 Cora Kohlmeier, Frank Hamberg

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#CEMoS C Environment for Model Simulation
#Control for Graphical user interface (CEvoS)
global GUI
set GUI 1
global toolflag
global cemhome cemoshome datahome cemospar netCDFHOME
global pwd home modelhome modelhomeabs model_path resultdir
global simname simtype outputtype
global allbuts startbuts firstread log pady
global DOS

global INSIGHT SHIFTCUT


global  mindelt maxdelt  method accuracy relrate relchange
set mindelt(1) 1
set maxdelt(1) 1
set accuracy(1) 0
set relrate(1)} 0
set relchange(1) 0
set method(1) 0
set resultdir .
set outputtype outc
##########################################################################
##########################################################################
set toolflag cemtk
set home $env(HOME)
set cemhome [file dir [info script]]
if {[file tail $cemhome] != "CEMTK"} {set cemhome $cemhome/CEMTK} 
set cemoshome $cemhome/../CEMOS 
set netCDFHOME $cemoshome/netcdf
set pwd1 "."

set pwd [pwd]
if {[lindex $argv 0] != ""} {set pwd1 [file dir [lindex $argv 0]]}
if {$pwd1!="."} {set pwd $pwd1} 

set model_path $pwd
if [file exists $pwd/cemos.par] {set cemospar cemos.par}

set resultdir $pwd

set ismacosapp 0
if [string match "*app\/Contents\/Resources*" $cemhome] {
	set ismacosapp 1
	} else {
	set moviehome $env(TKMOVIEHOME)
}
# alle procs laden
set dirlist "util cemos"

foreach item $dirlist {
  auto_mkindex $cemhome/$item *.tcl
  lappend autopath $cemhome/$item
  set dir $cemhome/$item
  set unit [open $cemhome/$item/tclIndex]
  eval [read $unit]
  close $unit
}

##########################################################################
##########################################################################
set_arch
# Preparing fileboxes (not necessary but nice for hidden files)
if {$DOS==0} {
   catch { tk_getOpenFile -w00t } ;# start file dialog with unknown option
       set ::tk::dialog::file::showHiddenBtn 1 
       set ::tk::dialog::file::showHiddenVar 0
} 

#erstmal Leichen beseitigen
if [file exists $pwd/integrat.par] {file delete $pwd/integrat.par}
if [file exists $pwd/.sensitivity.par] {file delete $pwd/.sensitivity.par}
if [file exists $pwd/.sensitivity.def] {file delete $pwd/.sensitivity.def}
if [file exists $pwd/.parfile.tmp] {file delete $pwd/.parfile.tmp}

if {$argc>0} {
 set cemospar [lindex $argv 0]
 if {[file tail $cemospar] == "cemos.par" } {
    set pwd1 [file dirname $cemospar]
    if {$pwd1!="."} {set pwd $pwd1} 
 }
}

set modelhomeabs $pwd/main

global colorscheme

#------------------------------
# global settings
#------------------------------
global lfont sfont
global geom xc
global command cancel
global bg c2
global editor
global general_flaglist gflagarr compile_flaglist  cflagarr
global comp_settings

set bg 0
set geom +150+0
set cancel 0
set allbuts ""

if ![file exist $home/.cem5defaults] {
	file copy $cemhome/cem5defaults $home/.cem5defaults
}
readdefaults
colorswitch
set defcol [option get . background {}]

#------------------------------
# Window manager configurations
#------------------------------
wm title . {Tiger Graphics CEMoS Control}
wm iconbitmap . @$cemhome/icons/tinylogo.pic
wm iconmask . @$cemhome/icons/tinylogo.pic

set screenheight [winfo screenheight .]

if {$screenheight < 1100} {
		set pady [expr [lindex $lfont 1] -4]
      wm geometry . "+150+0"
} else {
      wm geometry . "+150+50"
		set pady [expr [lindex $lfont 1] -2]
}
if {$pady < 0} {set pady 0}
# Liste der General compile flags
# werden in gflagarr zusammengefasst. Indexe sind die Elemente von  general_flaglist
set general_flaglist "g static ansi pedantic D__verbose__ D__accept_nega_with_zero_rate__ Wredundant-decls Wstrict-prototypes"
lappend general_flaglist Wextra Wshadow Wfloat-equal Wunreachable-code 
lappend general_flaglist   fsignaling-nans     
if {$DOS==0} { 
    lappend general_flaglist pg Wunused-result Wno-unused-result Wunused-but-set-variable Wno-unused-but-set-variable efence gsl
}
#Initialisierung der arrays
foreach item $general_flaglist {set gflagarr($item) 0}

# Liste der Modell compile flags
# werden in cflagarr zusammengefasst. Indexe sind die Elemente von  compile_flaglist
set compile_flaglist "O O2 O3 Wall"
if {$DOS==0} { 
    lappend compile_flaglist O4
}
#set compile_flaglist "O O2 O3 O4 Wall fopenmp"

#Initialisierung der arrays
foreach item $compile_flaglist {set cflagarr($item) 0}

make_default_list
#################################################################################

# Create a frame for buttons and entry.
frame .topp -bg $c2
pack .topp -side top -fill x -fill y
frame .topp.lab
grid .topp.lab -row 0 -column 0 -sticky news

# Initializing of the info window
#---------------------------------
execlog
# Initializing of variables
#---------------------------------

ReadPar
WritePar
setvars

set simname $resultdir/result

# Model Control
#----------------------------

labelframe .topp.mod -bg $c2 -text "Model Control"
grid .topp.mod -row 2 -column 0 -sticky news -padx 5 -ipadx 5 -ipady 5 -pady {0 5}

#label .topp.mod.head -text "Model Control" -font $lfont -bg $c2
#pack .topp.mod.head -fill x


label .topp.mod.pwd -text \
      "Current Dir: $pwd" \
           -font $lfont  -bg $c2 -anchor w -relief sunken
pack .topp.mod.pwd -side left -padx {5 0}

button .topp.mod.bro -text Browse -font $lfont  -bg $c2 \
                -command {setmodeldir;set firstread 1;ReadPar;WritePar;setmenu}
lappend allbuts .topp.mod.pwd .topp.mod.bro

pack .topp.mod.bro -side left -padx {10 0}

label .topp.mod.moh -text \
      "Model Dir:  $modelhome" \
           -font $lfont  -bg $c2 -anchor w -relief sunken
pack .topp.mod.moh -side left -padx 10
lappend allbuts .topp.mod.moh

button  .topp.mod.new -text {Create New Model} -font $lfont -bg $c2 \
-command {newmod}
pack  .topp.mod.new -side right -padx {0 5}
lappend allbuts .topp.mod.new


# Compiler Control
#----------------------------
# Compiler Control
#----------------------------
labelframe .topp.com -text "Compiler Control" 
grid .topp.com -row 3 -column 0 -sticky news -padx 5 -ipadx 5 -ipady 5 -pady {0 5}
button  .topp.com.but -text {Compiler Flags}  -font $lfont\
         -command {compilerflags}
pack  .topp.com.but -side left -padx {5 0}

#Default Flags auswaehlen
global cfmenu
#$cfmenu invoke default

button  .topp.com.cmp -text {Compile} -font $lfont \
-command {
          $log delete 1.0 end
          set gflags [make_generalflags]
          set cflags [make_compileflags]
          set lflags [make_linkflags]
          compile $modelhome $gflags $cflags $lflags
}
pack  .topp.com.cmp -side left

button  .topp.com.up -text {Update} -font $lfont \
-command {
         set gflags [make_generalflags]
         set cflags [make_compileflags]
         set lflags [make_linkflags]
         cup $modelhome $gflags $cflags $lflags
}
pack  .topp.com.up -side left


button  .topp.com.clr -text {Clean} -font $lfont -command {clean $modelhome}
pack  .topp.com.clr -side left

lappend allbuts .topp.com.cmp .topp.com.up .topp.com.clr .topp.com.but

if [file exists $pwd/$modelhome/install.tcl] {
   button  .topp.com.ui -text {Uninstall} -font $lfont -command {uninstall $modelhome}
   pack  .topp.com.ui -side left
   lappend allbuts .topp.com.ui
}

# Simulation Control
#----------------------------
labelframe .topp.sim -bg $c2 -text "Simulation Control"
grid .topp.sim -row 4 -column 0 -sticky news -padx 5 -ipadx 5 -ipady 5 -pady {0 5}

#label .topp.sim.head -text "Simulation Control" -font $lfont -bg $c2
#pack .topp.sim.head

checkbutton .topp.sim.bg -text {Background} -font $lfont -variable bg \
            -bg $c2 -relief raised
pack .topp.sim.bg -side left -padx {5 0}
lappend allbuts .topp.sim.bg

button .topp.sim.sim -text {Start !} -font $lfont  -bg $c2 \
-command {

          if [file exist .sensitivity.def] {exec rm -f .sensitivity.def}
          set cancel 0;
          set simname {};
          deactivate $allbuts
          startsim
          if {$cancel != 1} {
			  WritePar
	          if {$bg==1} {
	             cem_run integrat.par -bg
	             activate $allbuts
	          } else {
	             cem_run integrat.par
	            if {$simtype!="outa"} {.topp.misc.movtk configure -state normal}
	          }
          } else {
             activate $allbuts	  
		  }
     }
pack .topp.sim.sim -side right -padx 5

button .topp.sim.bat -text {Start Batch} -font $lfont  -bg $c2 \
-command {startbatch}
pack .topp.sim.bat -side right -padx 5 -ipadx 5 

button .topp.sim.cin -text {Start Run} -font $lfont  -bg $c2 \
-command {startcin}
pack .topp.sim.cin -side right -padx 5


activate_start

lappend allbuts .topp.sim.sim .topp.sim.bat .topp.sim.cin
set startbuts ".topp.sim.sim .topp.sim.bat .topp.sim.cin"

#------------------------------
# Set Integration parameters
#------------------------------
integpar

# Edit Files
#------------------
labelframe .topp.ed -text "File Actions"
grid .topp.ed -row 5 -column 0 -sticky news -padx 5 -ipadx 5 -ipady 5 -pady {0 5}

#label .topp.ed.head -text "Editor Control" -font $lfont
#pack .topp.ed.head -fill x


button  .topp.ed.rename -text {Rename Result File}  -font $lfont\
         -command {renameResult}
pack  .topp.ed.rename -side left -padx {5 0}

button  .topp.ed.new -text {New File} -font $lfont\
-command {
.topp.ed.new config -state disabled
newfile $modelhomeabs
.topp.ed.new config -state normal
}
pack  .topp.ed.new -side left
lappend allbuts .topp.ed.new

button  .topp.ed.code -text {Edit Code Files} -font $lfont\
-command {
.topp.ed.code config -state disabled
editfiles "Select code file" $modelhomeabs c
.topp.ed.code config -state normal
}
pack  .topp.ed.code -side left


button  .topp.ed.par -text {Edit Parameter Files} -font $lfont\
-command {
.topp.ed.par config -state disabled
editfiles "Select parameter file" $modelhomeabs def
.topp.ed.par config -state normal
}
pack  .topp.ed.par -side left

if {$DOS==1} {
button  .topp.ed.log -text {Edit Log Files} -font $lfont\
-command {
.topp.ed.log config -state disabled
editfiles "Select .log file" $pwd log
.topp.ed.log config -state normal
}
pack  .topp.ed.log -side left
}

button  .topp.ed.cin -text {Edit Run Files} -font $lfont\
-command {
#eval exec $editor $pwd/sensitivity.bat &
.topp.ed.cin config -state disabled
editfiles "Select Run file" $pwd cin
.topp.ed.cin config -state normal
}
pack  .topp.ed.cin -side left

button  .topp.ed.bat -text {Edit Batch Files} -font $lfont\
-command {
#eval exec $editor $pwd/sensitivity.bat &
.topp.ed.bat config -state disabled
editfiles "Select Batch file" $pwd bat
.topp.ed.bat config -state normal
}
pack  .topp.ed.bat -side left


button  .topp.ed.sim -text {Edit Simulation Parameters} -font $lfont\
-command {
.topp.ed.sim config -state disabled  

if {$DOS==1} {eval exec {$editor $pwd/cemos.par}}
if {$DOS==0} {
	if {$editor=="gedit"} {
	  eval exec {gedit -s $pwd/cemos.par 2>/dev/null}
	} else {
	  if {$editor=="kate"} {
	     eval exec {kate --startanon $pwd/cemos.par 2>/dev/null}
	  } else {
	     eval exec {$editor $pwd/cemos.par 2>/dev/null}
	  }
	}
}
if {$DOS==2} {
	if {$defaulteditor==""} {
		set command "open -e $pwd/cemos.par"
		eval exec $command 
	} else {
	    eval exec {$editor $pwd/cemos.par 2>/dev/null}
	}
}
update idletasks

ReadPar;WritePar;setmenu

.topp.ed.sim config -state normal
}
pack  .topp.ed.sim -side left

lappend allbuts .topp.ed.rename .topp.ed.code .topp.ed.par .topp.ed.log .topp.ed.sim .topp.ed.bat .topp.ed.cin

# Miscs
#------------------
#labelframe .topp.misc -bg $c2 -text "Miscellaneous"
frame .topp.misc -bg $c2
grid .topp.misc -row 6 -column 0 -sticky news -padx 5 -ipadx 5 -ipady 5 -pady {0 5}

button  .topp.misc.license -text License -font $lfont \
        -command {deactivate $allbuts;deactivate $startbuts;tatzetool license} -bg $c2
pack  .topp.misc.license -side left -padx {5 5}
lappend allbuts .topp.misc.license


button  .topp.misc.custom -text Options -font $lfont -command {deactivate $allbuts;deactivate $startbuts;
          .topp.ed.sim config -state disabled;
          customtool
	  } -bg $c2
pack  .topp.misc.custom -side left -padx 5
lappend allbuts .topp.misc.custom

button  .topp.misc.tatze -bitmap @$cemhome/icons/tinylogo.pic \
        -command {deactivate $allbuts;deactivate $startbuts;tatzetool cemos} -bg $c2
pack  .topp.misc.tatze -side right -padx {0 0}
lappend allbuts .topp.misc.tatze

button  .topp.misc.exit -text {   Exit  }  -font $lfont \
   -command {destroy .
              if [file exists $pwd/integrat.par] {file delete $pwd/integrat.par}
              if [file exists $pwd/.sensitivity.par] {file delete $pwd/.sensitivity.par}
              if [file exists $pwd/.sensitivity.def] {file delete $pwd/.sensitivity.def}
              if [file exists $pwd/.parfile.tmp] {file delete $pwd/.parfile.tmp}

#	     if {$tclstop == 0} {
#	        Stopmodel}
            } -bg $c2
pack  .topp.misc.exit  -side right -fill x -padx 5
lappend allbuts .topp.misc.exit

set wishcmd wish
button  .topp.misc.movtk -text {Tiger Graphics MoViE} -font $lfont -command {
#                                  .topp.com.up invoke
                 if {$ismacosapp==1} {
					 set command "open -a MoViE --args $simname.$simtype"
					 puts $command
                     eval exec $command
				 } else {
					 eval exec $wishcmd $moviehome/mov.tcl $simname.$simtype &
				 }
			 } -bg $c2
pack  .topp.misc.movtk  -side right -fill x -padx 5

if {$SHIFTCUT==1} {
button  .topp.misc.sc -text {ShiftCut} -font $lfont -command {
                                  .topp.com.up invoke
								  deactivate $allbuts
                                  update
                                  update idletasks
                                  shiftcut $simname
                                  puts "Ready Sc"
                                  set simname $simname\_shift\_cut
								  activate $allbuts
								  .topp.misc.sc configure -state disabled
                                  update
                                  update idletasks
                                  } -bg $c2
				 
   pack  .topp.misc.sc  -side right -fill x -padx 5
   lappend allbuts .topp.misc.sc
}


if {$DOS==0 || $DOS==2} {
   button  .topp.misc.terminal -text "Terminal" -font $lfont -command {OpenTerm} -bg $c2
   pack  .topp.misc.terminal -side right -padx 5
   lappend allbuts .topp.misc.terminal
}

button  .topp.misc.editor -text "Editor" -font $lfont -command {OpenEdit} -bg $c2
pack  .topp.misc.editor -side right -padx 5
lappend allbuts .topp.misc.editor

lappend allbuts .topp.misc.movtk
if {$DOS==0 || $DOS==2} {
   button .topp.misc.db -text {DDD} -font $lfont -bg $c2 -command {.topp.com.up invoke;exec ddd cemos-model > /dev/null &}
   pack  .topp.misc.db -side right -padx 5
   lappend allbuts .topp.misc.db

   if {$INSIGHT==1} {
   button .topp.misc.ins -text {Insight} -font $lfont -bg $c2 -command {.topp.com.up invoke;exec insight cemos-model > /dev/null &}
   pack  .topp.misc.ins -side right -padx 5
   lappend allbuts .topp.misc.ins
   }
}

ReadPar
WritePar
setvars
#update
#puts " [winfo reqwidth .sim]  [winfo reqwidth .ed] [winfo reqwidth .topp.paned]"
#.topp.paned configure -width [winfo reqwidth .ed]
#update
#puts " [winfo reqwidth .sim]  [winfo reqwidth .ed] [winfo reqwidth .topp]"

. configure -bg $c2
if {![file exists $pwd/cemos-model]} {
   put "No executable cemos-model found!"
   put "Please (re-)compile the model."
   activate $allbuts
   activate_start
}
update
update idletasks
