#!/bin/sh
# the next line restarts using wish\
exec wish "$0" "$@"
# CEMOS/CEMTK 5.0 Copyright(C) 2022 Cora Kohlmeier, Frank Hamberg

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

#For details see License Button.

#CEMoS Evolution System

# Control for starting cev with graphical user interface or not (batch mode)
global GUI DOS ismacosapp
global toolflag
global cemhome cemoshome datahome cemospar netCDFHOME
global pwd home modelhome modelhomeabs model_path
global simname simtype
global cev_resultdir cev_absbestdir
global absbestdist distance
global counter changecounter broken oldtype difftype
global multiruns outputtype colorscheme

set GUI 0
set counter 0
set changecounter 0
set multiruns 0
set distance inf
set broken 0

set cev_cin_name ""
set cev_file ""
set cev_reference_file ""
set outputtype outc
##########################################################################
##########################################################################
# Betriebssystem ermitteln
set DOS 0
set arch [exec uname -s]
if {[string match CYGWIN* $arch]==1} {set DOS 1}
if {[string match Darwin* $arch]==1} {set DOS 2}

# Preparing fileboxes (not necessary but nice for hidden files)
if {$DOS==0} {
   catch { tk_getOpenFile -w00t } ;# start file dialog with unknown option
       set ::tk::dialog::file::showHiddenBtn 1 
       set ::tk::dialog::file::showHiddenVar 0
} 

##########################################################################
##########################################################################
set toolflag cevos
set home $env(HOME)
set cemhome [file dir [info script]]
if {[file tail $cemhome] != "CEMTK"} {set cemhome $cemhome/CEMTK} 
set ismacosapp 0
if [string match "*app\/Contents\/Resources*" $cemhome] {
	set ismacosapp 1
	} else {
	set moviehome $env(TKMOVIEHOME)
}
set cemoshome $cemhome/../CEMOS 
set netCDFHOME $cemoshome/netcdf
set pwd1 "."

set pwd [pwd]

# Um den richtigen Pfad auch aus dem Dateimanager zu kriegen
if {[lindex $argv 0] != ""} {set pwd1 [file dir [lindex $argv 0]]}
if {$pwd1!="."} {set pwd $pwd1} 

#erstmal Leichen beseitigen
if [file exists $pwd/integrat.par] {file delete $pwd/integrat.par}
if [file exists $pwd/.sensitivity.par] {file delete $pwd/.sensitivity.par}
if [file exists $pwd/.sensitivity.def] {file delete $pwd/.sensitivity.def}
if [file exists $pwd/.parfile.tmp] {file delete $pwd/.parfile.tmp}

set model_path $pwd
set simname $pwd/result
set modelhomeabs $pwd/main

# alle procs laden
set dirlist "util cevutil cemos"

foreach item $dirlist {
  auto_mkindex $cemhome/$item *.tcl
  lappend autopath $cemhome/$item
  set dir $cemhome/$item
  set unit [open $cemhome/$item/tclIndex]
  eval [read $unit]
  close $unit
}

# Hilfe
if {[lindex $argv 0]=="--help"} {
      puts "Usage Batch mode:"
      puts "cev:  reads file cevos.init if exists"
      puts "cev --reset             :resets cevos"
      puts ""
      puts "Usage GUI mode:"
      puts "cev cemos.par"
      exit
}

#Reset
set cev_resultdir $pwd/cevos
set cev_absbestdir $pwd/cevos_absbest

if {[lindex $argv 0]=="--reset"} {
	 set oldtype  0
	 set difftype 1
     cev_reset
     exit
}

#Aufruf ueber Konqueror
if {$argc==1} {
     set cemospar [lindex $argv 0]
      if {[file tail $cemospar] == "cemos.par" } {
          set pwd1 [file dirname $cemospar]
          if {$pwd1!="."} {set pwd $pwd1} 
          set GUI 1
      } else {
          exit
      }
}

if [file exists $pwd/cemos.par] {set cemospar cemos.par} else {puts "No file cemos.par";exit}
if [file exist $pwd/cevos] {set cev_resultdir $pwd/cevos} else {puts "No cevos directory";exit}

cev_randomInit [pid]
# Diagnose cevos_testmodel2 broken im ersten Lauf
#cev_randomInit 16497

puts [pid]
##########################################################################
#Einlesen der Default-Konstellation in setvars und
#Uebersteuerung der Settings mittels Datei cevos.init in cev_resultdir  
cev_setvars

##########################################################################
#Einlesen der absolut besten Distanz, falls existent

set file $cev_resultdir/absbest.dist
if  [file exists $file] {
	set id [open $file r]
    while {[gets $id line] >=0} {
    eval $line
    }
    close $id

}

#Einlesen der Counter
cev_read_count

if ![file exist $cev_file] {
      put "Parameter file $cev_file does not exist"
      put "Please select .cev file"
      if {$GUI==0} {exit}
}

if [file exist $cev_file] {
      cev_readpar $cev_file
      cev_init_best
      cev_check_best
#cev_reset_tobest
      if {$GUI==0} {cev_reset_tobest}
      cev_write_sensitivitybat
      cev_write_count
     if {$GUI==0} {cev_start_random}
}

#Uebergordenete settings aus cemtk
setvars
if {$GUI==1} {cev_gui_make_window}



