#!/usr/bin/tclsh
# the next line restarts using wish\
# exec wish "$0" "$@"
# CEMOS/CEMTK 4.1.1 Copyright(C) 2022 Cora Kohlmeier, Frank Hamberg

# CEMoS Evolution System for Models with random numbers
# Starting only from terminal
# with arguments <number of random sessions> and <runs per session> 
# without argument: one session until manually stopped (same as cev)

####################################################################
# the model must evaluate srand(cem_seed)
# the model needs a variable  cem_seed in model.def AND in %change of cemos.par
# the %change block of cemos.par is adapted by this program!!!
####################################################################

# Control for starting cev with graphical user interface or not (batch mode)
global GUI
global DOS

global toolflag
global cemhome cemoshome moviehome datahome cemospar netCDFHOME
global pwd home modelhome modelhomeabs model_path
global simname simtype

global cev_resultdir 
global cev_absbestdir
global absbestdist
set GUI 0
global counter
set counter 0
global rndcounter
set rndcounter 0
global multiruns
set multiruns 0

set cev_cin_name ""
set cev_file ""
set cev_reference_file ""

##########################################################################
##########################################################################
# Betriebssystem ermitteln
set DOS 0
set arch [exec uname -s]
if {[string match CYGWIN* $arch]==1} {set DOS 1}
if {[string match Darwin* $arch]==1} {set DOS 2}

##########################################################################
##########################################################################
set toolflag cevos
set home $env(HOME)
set cemhome $env(TKCEMHOME)
set cemoshome $env(CEMOS)
set moviehome $env(TKMOVIEHOME)
set netCDFHOME $cemoshome/netcdf
set pwd [pwd]

# Um den richtigen Pfad auch aus dem Dateimanager zu kriegen
#if {[lindex $argv 0] != ""} {set pwd [file dir [lindex $argv 0]]}

#erstmal Leichen beseitigen
if [file exists $pwd/integrat.par] {file delete $pwd/integrat.par}
if [file exists $pwd/.sensitivity.par] {file delete $pwd/.sensitivity.par}
if [file exists $pwd/.sensitivity.def] {file delete $pwd/.sensitivity.def}
if [file exists $pwd/.parfile.tmp] {file delete $pwd/.parfile.tmp}

set model_path $pwd
set simname $pwd/result
set modelhomeabs $pwd/main

# alle procs laden
set dirlist "util mocutil cevutil cemos"

foreach item $dirlist {
  auto_mkindex $cemhome/$item *.tcl
  lappend autopath $cemhome/$item
  set dir $cemhome/$item
  set unit [open $cemhome/$item/tclIndex]
  eval [read $unit]
  close $unit
}

# Hilfe
if {[lindex $argv 0]=="--help"} {
      puts "Usage:"
      puts "cevmulti.tcl <anzahl random sessions> <runs per session>:  reads file cevos.init if exists"
      puts ""
      exit
}

#Reset
set cev_resultdir $pwd/cevos
set cev_absbestdir $pwd/cevos_absbest
#cev_reset

#Aufruf mit Anzahl der sessions (=1 ein Lauf bis maxruns)
if {$argc>1} {
     set anzahl [lindex $argv 0]; # number of sessions
     set multiruns 1;# switch for multiruns
     set runspersession [lindex $argv 1]; # number of runs per session
} else {
    set anzahl 1;# one session (same as cev)
    set multiruns 0;# runs until manually stopped
}

if [file exists $pwd/cemos.par] {set cemospar cemos.par} else {puts "No file cemos.par";exit}
if [file exist $pwd/cevos] {set cev_resultdir $pwd/cevos} else {puts "No cevos directory"; exit}

cev_randomInit [pid]

##########################################################################
#Einlesen der Default-Konstellation in setvars und
#Uebersteuerung der Settings mittels Datei cevos.init in cev_resultdir  
cev_setvars
if ![file exist $cev_file] {puts "File $cev_file does not exist"; exit}

# multi directory sichern
set date [clock format [clock seconds] -format "%Y%m%d_%H%M%S" ]
if ![file exist $pwd/cevos_multi_dirs] {file mkdir $pwd/cevos_multi_dirs} 
if [file exist $pwd/cevos_multi] {file copy $pwd/cevos_multi $pwd/cevos_multi_dirs/cevos_multi_$date} 

# neues leeres Verzeichnis cevos_multi anlegen
if [file exist $pwd/cevos_multi] {file delete -force $pwd/cevos_multi} 
if ![file exist $pwd/cevos_multi] {file mkdir $pwd/cevos_multi} 

# absbest_mean sichern
if [file exist $pwd/absbest_mean.cin] {file copy $pwd/absbest_mean.cin $pwd/cevos_multi_dirs/absbest_mean_$date.cin} 

# Loop over multiruns
if {$multiruns==1} {set maxruns $runspersession}
for {set k 1} {$k<=$anzahl} {incr k} {
    cev_multi_adapt_cemospar
    cm_readpar $cev_file
    cev_reset
    cev_write_sensitivitybat
    cev_start_random
    puts "Session $k of $anzahl ready"

    if [file exist $pwd/cevos/absbest.cin] { file copy -force $pwd/cevos/absbest.cin $pwd/cevos_multi/absbest_$k.cin }
    
}
cev_multi_mean $pwd/cevos_multi
puts "Ready"


