/* MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg */

#include "stdinc.h"

#define rem1 "/*"
#define rem2 "*/"

int main(int argc, char *argv[])
{
 int get_data(char*,char*,char* ,real *);
 void data_analyse(int ,float*, int , real*,real*,real*);
 void writeheader(FILE *, header*, long_name80*, int);
 void interpolate(real*, real, real, int , real*, real*, int);
 char *symbname, *input, *helpstr;
 string input_file,output_file,para;
 int timnum=0;
 int i,j,k,l,m,n,cp,len;
 int num_of_years,numsteps;
 unsigned int nvars=0;
 int *num_data,count;

 float t_cycle;

  int check1,check2,check3,check4,namelength;
  int item,dyear,month,day,hour,minute;
  long_name svar,last_var,helpname;
  long_name80 *all_var_names;
  float *dat,varianz;
  real argu;
  real *var_dat,*var_ymd,*var_tid,*var_tim,*time_vec;
  real **all_data,**target_data;
  real *var_data, *base_data;

  FILE *in_file, *out_file;

  float monthlen[13]={0,31,28,31,30,31,30,31,31,30,31,30,31};
  float monthtim;
  float offset;
  float fillvalue;
  header oh;

  namelength=sizeof(long_name80);
  oh.sdir = malloc(sizeof(char)*namelength);

  input=   (char *) malloc( _BUFSIZE_*sizeof(char));
  helpstr=(char *) malloc( _BUFSIZE_*sizeof(char));

/* Vorlaeufig 'mal was fuer die Variablen-Namen reservieren */
  all_var_names = malloc(one_var*namelength);
  num_data = malloc(one_var*sizeof(int));
  last_var[0] = (char) 0;

/* Check the symb file and get the ranges */

  symbname=argv[1];

  sprintf(input_file,"%s.symb",symbname);
  sprintf(output_file,"%s_symb.outc",symbname);

  in_file=fopen(input_file,"r");

  if (in_file == NULL)
  {
     std_error("Error when opening data file for reading");
  }
/*
  Es koennen Vorgabewerte fuer folgende Daten im Header des *_symb.outc-Files angegeben werden:
  starttime: Zeitwert fuer den ersten Wert
  endtime: Zeitwert fuer den letzten Wert
  cycle     : Zyklusvorgabe 
  outdelt   : Vorgabe fuer das outdelt 

  Wird nichts uebergeben, gilt:
  cycle     = 360.0
  outdelt   = 1.0
  starttime = 0.0
  endtime = cycle
*/
  oh.cycle     = 360.0;
  oh.outdelt   = 1.0;
  oh.startim  = 0.0;  
  oh.endtim  = oh.cycle;  

   offset=0.0;
   fillvalue=0.0;

  if (argc > 1)
  {
/*   Vorgabe fuer startim einlesen    */
     sprintf(para,"%s",argv[2]);
     sscanf(para,"%f",&oh.startim);

/*   Vorgabe fuer endtim einlesen    */
     sprintf(para,"%s",argv[3]);
     sscanf(para,"%f",&oh.endtim);

/*   Vorgabe fuer cycle einlesen    */
     sprintf(para,"%s",argv[4]);
     sscanf(para,"%f",&oh.cycle);

/*   Vorgabe fuer outdelt einlesen    */
     sprintf(para,"%s",argv[5]);
     sscanf(para,"%f",&oh.outdelt);

/*   Vorgabe fuer offset einlesen    */
     sprintf(para,"%s",argv[6]);
     sscanf(para,"%f",&offset);

/*   Vorgabe fuer fillvalue einlesen    */
     sprintf(para,"%s",argv[7]);
     sscanf(para,"%f",&fillvalue);



   }

  while ( fgets(input,_BUFSIZE_,in_file) != 0 )
  {
     cp = -1;
     sprintf(helpstr,"%s",input);
     len= strlen(helpstr);
     if (len>5) //Leerzeilen filtern, mindestens x/xxx/: muss drin stehen
     {
        cp = strpos(strmid(helpstr,len-2,1),",");
        count = numchar(helpstr,",");
        while ( cp >= 0)
        {
            fgets(helpstr,_BUFSIZE_,in_file);
            len= strlen(helpstr);
            cp = strpos(strmid(helpstr,len-2,1),",");
            count += numchar(helpstr,",");
        }
        cp = strpos(input,")");
        if ( cp > 0)
        {
            sprintf(svar,"%s",strmid(input,0,cp+1));
            if (strpos(last_var,svar) < 0)
            {
            sprintf(helpname,"%s",strmid(svar,0,namelength));
            for (m=(int)strlen(helpname);m<namelength;m++) helpname[m]= (char) 32;
            sprintf(all_var_names[nvars],"%s",strmid(helpname,0,namelength));
            sprintf(last_var,"%s",svar);
            num_data[nvars] = count+1;  /*Anzahl der Werte ist Anzahl der Kommas + 1 */
            timnum = IMAX(timnum,count+1);
            nvars++;
            }
        }
     }
  }
  fclose(in_file);

/* Speicherplatz fuer eine Variable (wird auf Maximallaenge angelegt) */
/* Daten*/
   var_dat =vector(1,timnum);
/* YearMonthDay*/
   var_ymd=vector(1,timnum);
/* Uhrzeit  */
   var_tid=vector(1,timnum);
/* Zeit in Zahlen */
   var_tim=vector(1,timnum);
/* Zeitvektor fuer Transformation */
   time_vec=vector(1,timnum);
/* Anzahl der zu erzeugenden Zeitschritte */
  numsteps = 1 + (oh.endtim-oh.startim)/oh.outdelt;

/* Feld fuer Zieldaten anlegen */
  target_data = matrix(1,nvars,0,numsteps);
  var_data     = vector(0,numsteps);

/* Messdaten einlesen */
/* Hauptschleife ueber alle Variablen */
  for (k=1;k<=nvars;k++)
  {
    sprintf(svar,"%s",strmid(all_var_names,(k-1)*namelength,namelength));
    strtrim(svar);
/* Eine Variable einlesen und aufpassen, dass ab Index 1 geschrieben wird*/
     check1=get_data(input_file,svar,"val",&var_dat[1]);
     check2=get_data(input_file,svar,"ymd",&var_ymd[1]);
     check3=get_data(input_file,svar,"tid",&var_tid[1]);
     check4=get_data(input_file,svar,"tim",&var_tim[1]);
/*
Ueberpruefen ob die Datenmengen stimmen, check3 wird ausgewertet, wenn Uhrzeiten verwertet werden
*/
     if ((check2!=0 && (check1 != check2 || check1 != num_data[k-1] || check2 != num_data[k-1] )) || (check2 == 0 && check1 != check4) )
                       std_error("Number of data and time samples are different");

/*Eigentlicher Indexzaehler j (hier wird verhindert, dass die ausgezeichneten Daten var_dat[i]=-9999 beruecksichtigt werden, welche mit m gezaehlt werden) */
      j = 1;
      m = 0;
      for (i=1;i<=num_data[k-1];i++)
      {
         if (var_dat[i]!=-9999.0)
         {
	   if (check4!=num_data[k-1])
	   {
/*      Jahr, Monat und Tag aus den Datensatz holen*/
             item		= (int) var_ymd[i];
             dyear		= (int) item/10000;
	     item		-=10000*dyear;
	     month		= (int) item/100;
	     item		-= 100*month;
	     day		= item;
/*Wenn in var_tid keine Werte angekommen sind, wird auf 12:00 gesetzt	 */
	     hour		= 12;
	     minute		= 0;
	     if ( check3 == check1 )
	     {
	        item 		= (int) var_tid[i];
	        hour		= (int) item/100;
	        item		-= 100*hour;
	        minute	= item;
	     }
             monthtim=0.0;
             for (l=0;l<month;l++) monthtim+=monthlen[l];
	     time_vec[j] = (monthtim + day + hour/24.0 + minute/1440.0)+offset;
          }
	  else
	  {
	    time_vec[j] = var_tim[i]+offset;
	  }
	 var_dat[j] = var_dat[i];
	 j++;
	 }
	 else
	 {
	 m++;
	 }
       }

   if (j>1)
   {
        /*Daten analysieren*/
        printf("interpolating %s [total=%d, invalid=%d, valid=%d] \n",svar,num_data[k-1],m,j-1);
        fflush(stdout);
        
        /*Rueckgabevektor loeschen*/
        for (i=0;i<=numsteps;i++) var_data[i]=0.0;
        
        /*Die wirkliche Anzahl Datensaetze ist j-1*/
        interpolate(var_data,oh.startim,oh.outdelt,numsteps,time_vec,var_dat,j-1);
   }
   else
   {
        /*Daten analysieren*/
        printf("filling %s [total=%d, invalid=%d, valid=%d]  with %f\n",svar,num_data[k-1],m,j-1,fillvalue);
        fflush(stdout);
        
        /*Rueckgabevektor mit fillvalue fuellen*/
        for (i=0;i<=numsteps;i++) var_data[i]=fillvalue;

   }
/*Daten sichern*/
   for (i=0;i<=numsteps;i++) 
  {
  if (var_data[i]!=-9999.0)
          target_data[k][i] = var_data[i];
  else 
          target_data[k][i] = fillvalue;

   }

  }

  dat = malloc(nvars*sizeof(float));
/* Header fuer output file setzen */

  oh.nvars= nvars;
  oh.maxdelt = oh.outdelt;
  oh.year = 0.0;
  strncpy(oh.sdir,"symb-file   ",12);
  for (m=12;m<namelength;m++) oh.sdir[m]= (char) 32;
  oh.sdir[namelength-1]= (char) 0;

  out_file=fopen(output_file,"wb");

 /* Header schreiben */
  writeheader(out_file,&oh,all_var_names,namelength);

  for (j=0;j<=numsteps;j++)
     {
     for (i=1;i<=nvars;i++) dat[i-1] = (float) target_data[i][j];
     fwrite(&dat[0],sizeof(float),nvars,out_file);
     }

  fclose(out_file);

}
