# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc compmean {j} {
global can mf
global timewin
global actstart
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global colorlist colorindex
global plotFont boldFont
global command
global graph gl
global vertoff
global filename filelist
global singleresult
global legendtext
global lbar
global dataname datalist multiset
global valu
global vtim vdat
global xvgr_flag matlab_flag gnu_flag matlab_header

global fixymax fixymin  prettyaxis

set c $mf.left.plot 
#set selectvar [lindex $command($j) 0]

set nbar [llength $command($j)]

set graph [lindex $gl $j]

set ymin($j) [min $ymin($j) 0]

if { [regexp {[^0-9\.-]} $fixymin ] == 1 }  {set fixymax ""}

if {$fixymin !=""}  {
  set ymin($j) $fixymin
} else {
  set ymin($j) [min $ymin($j) 0]
}

if { [regexp {[^0-9\.-]} $fixymax ] == 1 }  {set fixymax ""}

if {$fixymax !=""}  {
  set ymax($j) $fixymax
} else {
  if {$prettyaxis==1} {
   set ymax($j) [max [rd $ymax($j)] 0]
  } else {
   set ymax($j) [max $ymax($j) 0]
  }
}

if {$ymin($j) >= $ymax($j)} {
   set ymax($j) [expr $ymin($j)+1.0]
   if {$ymin($j) > $ymax($j)} {textinfo "ymax<ymin !!"}

}

# Setting of x-axis
set xmin $actstart(1)
set xmax [expr $xmin + $timewin]

set xdelt [expr {$xmax -$xmin}]

# Plotting of x-axis and y axis
#--------------------------
set item [$c create line $pxmin $pymin $pxmin $pymax -width 1.5]
$c addtag axis withtag $item
$c addtag $graph withtag $item

set item [$c create line $pxmin $pymin $pxmax $pymin -width 1.5]
$c addtag axis withtag $item
$c addtag $graph withtag $item

if {$multiset==1} {
   set pxoff_label [expr $pxdelt/$nbar]
   } else {
   set pxoff_label [expr $pxdelt]
   }
set pxlabel [expr $pxmin + $pxoff_label/2]

set pxoff [expr $pxdelt/$lbar]
set px1 $pxmin
set px2 [expr $pxmin + $pxoff]

# Plotting of header
#--------------------------

scan $xmin %u xtext1
scan $xmax %u xtext2

set header "Time: $xtext1 - $xtext2"
set item [$c create text [expr $pxmin+20] 30 -text $header \
           -font $plotFont -anchor w]
$c addtag axis withtag $item
$c addtag $graph withtag $item


#set header [makeheader $selectvar]
#set item [$c create text [expr $pxmin+20] 60 -text $header \
#           -font $plotFont -anchor w]
#$c addtag axis withtag $item
#$c addtag $graph withtag $item


# Plotting of y-axis labels
#--------------------------
set ydelt [expr $ymax($j) - $ymin($j)]

for {set i 0} {$i <= 6} {incr i} {
   set ylabel [expr $ymin($j) + ($i*$ydelt/6)]
   set ytext [format "%10.3g" $ylabel]
   set yoff [nnyj $ylabel $j]

   set item [$c create line [expr $pxmin-5] $yoff $pxmin $yoff -width 2]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item
   set item [$c create text [expr $pxmin-10] $yoff -text $ytext \
                   -anchor e -font $plotFont]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item
}
#$c bind axis <3> "erase $c axis"

# Setting of legendtext
#----------------------
make_legendtext $j

if {$xvgr_flag == 1} { xv_open_graph $j  0 [expr $nbar+1] \
                                      $ymin($j) $ymax($j) $header bar
                        }

  if {$matlab_flag == 1 } {matlab_open_graph $j 0 [expr $nbar+1] \
                                      $ymin($j) $ymax($j) $header bar
                        }


# Sonderfunktion fuer Multiset-Barcharts
#--------------------------------------

if {$xvgr_flag ==1 && $multiset==1} {
   set index 1
   foreach selectvar $command($j) {
      xv_set_tickmark $index $selectvar
      incr index
   }
   set setindex 0
   foreach k $filelist {
     xv_open_multibar_header $j $setindex $legendtext($k)
     set index 1
     foreach selectvar $command($j) {
       set pmean $singleresult($k,$selectvar)
       xv_set_multibar $index $pmean
       incr index
     }
   xv_close_multibar_header
   incr setindex
   }
}
#--------------------------------------

set numsets [llength $command($j)]

set px2 [expr $pxmin + $pxoff/$numsets]

set setindex 0

foreach selectvar $command($j) {
set colorindex 0
foreach k $filelist {

   # Add variable name to legend text, if multiset is on
   #---------------------------------------------------
   if {$multiset == 1} {
      set legend "$selectvar in $legendtext($k)"
   } else {
      set legend $legendtext($k)
   }
   # Plotting of legend
   #-------------------
   set color [lindex $colorlist $colorindex]
   set row [expr 20+$vertoff*$setindex]
   if {$j == 0} {


    set item [$c create text 450 $row -text $legend \
             -font $plotFont -fill $color -anchor w]
    $c addtag $color$graph withtag $item
    $c addtag legend$graph withtag $item
    $c addtag $graph withtag $item

    $c bind legend$graph <1> "complegendDown $c %x %y legend$graph;\
                           $c itemconfig legend$graph -font {$boldFont}"
    $c bind legend$graph <ButtonRelease-1> "complegendMove $c %x %y;\
                           $c dtag selected;\
                           $c itemconfig legend$graph -font {$plotFont}"
    set compleg(lastX) 0
    set compleg(lastY) 0
   }

  # Plotting of data
  #-----------------
   set pmean [nnyj $singleresult($k,$selectvar) $j]

   # Plotting one bar
   #------------------------

   set item [eval {$c create poly} \
          $px1 $pymin $px1 $pmean $px2 $pmean $px2 $pymin  \
               { -width 1 -fill $color}]
   $c addtag $color$graph withtag $item
   $c addtag $graph withtag $item
   $c bind $color$graph <3> "set eraseset 1;makecommand -1 -1 $colorindex $j"

   set px1 $px2
   set px2 [expr $px2 + $pxoff/$numsets]

    if {$xvgr_flag ==1 && $multiset==0} {
       set pmean $singleresult($k,$selectvar)
       xv_set_bar [expr $setindex] $pmean $legend
    }

    if {$matlab_flag ==1 && $multiset==0} {
       set pmean $singleresult($k,$selectvar)
       matlab_add_bar [expr $setindex] $pmean  $legend
    }

    if {$gnu_flag ==1 && $multiset==0} {
       set pmean $singleresult($k,$selectvar)
       gnu_add_bar [expr $setindex] $pmean  $legend
    }

    incr setindex
    incr colorindex
}
    set item [$c create text $pxlabel [expr $pymin+10] -text $selectvar \
                       -anchor n -font $plotFont -fill [lindex $colorlist 0]]
    $c addtag $color$graph withtag $item
    $c addtag $graph withtag $item
    set pxlabel [expr $pxlabel + $pxoff_label]

}

#############################
# PLOTTING OF VALIDATION DATA
#############################

if {$valu ==1} {
#set colorindex [max 3 [lindex $filelist end]]

foreach a $datalist {
  if [info exists vtim($a,$j)] {
     set color [lindex $colorlist $colorindex]

     set pmean [nnyj $vdat($a,$j) $j]

     set item [eval {$c create poly} \
          $px1 $pymin $px1 $pmean $px2 $pmean $px2 $pymin  \
               {-width 1 -fill $color}]
     $c addtag $graph withtag $item

   set px1 $px2
   set px2 [expr $px2 + $pxoff]

   # Plotting of legend
   #-------------------
   set row [expr $row+$vertoff]
   if {$j == 0} {

    set text [file rootname [file tail $dataname($a)]]
    set item [$c create text 450 $row -text $text \
             -font $plotFont -fill $color -anchor w]
    $c addtag $color$graph withtag $item
    $c addtag legend$graph withtag $item
    $c addtag $graph withtag $item

    $c bind legend$graph <1> "complegendDown $c %x %y legend$graph;\
                           $c itemconfig legend$graph -font {$boldFont}"
    $c bind legend$graph <ButtonRelease-1> "complegendMove $c %x %y;\
                           $c dtag selected;\
                           $c itemconfig legend$graph -font {$plotFont}"

    set compleg(lastX) 0
    set compleg(lastY) 0
   }
    if {$xvgr_flag ==1 && $multiset==0} {
       xv_set_bar $setindex $vdat($a,$j) \
                  [file rootname [file tail $dataname($a)]]
    }

    if {$matlab_flag ==1 && $multiset==0} {
       matlab_add_bar $setindex $vdat($a,$j) \
                  [file rootname [file tail $dataname($a)]]
    }

    if {$gnu_flag ==1 && $multiset==0} {
       gnu_add_bar $setindex $vdat($a,$j) \
                  [file rootname [file tail $dataname($a)]]
    }

    incr setindex
    incr colorindex

  }
  }
}

if {$matlab_flag ==1} {
   set matlab_header "[makeheader $selectvar]   Mean over $header"
    matlab_plot_bars
}

if {$gnu_flag ==1} {
    set gnu_header "[makeheader $selectvar]   Mean over $header"
    gnu_plot_bars $j $gnu_header
}

}


