# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc axes {j {c none}} {

global can mf
global outdelt timewin cycle
global actstart
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global colorlist colorindex
global plotFont boldFont
global command
global graph gl singleresult
global vertoff
global xvgr_flag spectrum
global smooth
global fixymax fixymin
global xax barchart
global month1 month3
global harm startharm
global logflag
global xgrid ygrid blue_ymin blue_ymax

if {$c=="none"} {set c $mf.left.plot} 
set blue_ymin $ymin(0)
set blue_ymax $ymax(0)

set xaxformat "%6.2f"

set month1(0)   J
set month1(30)  F
set month1(60)  M
set month1(90)  A
set month1(120) M
set month1(150) J
set month1(180) J
set month1(210) A
set month1(240) S
set month1(270) O
set month1(300) N
set month1(330) D

set month3(0)   Jan
set month3(30)  Feb
set month3(60)  Mar
set month3(90)  Apr
set month3(120) May
set month3(150) Jun
set month3(180) Jul
set month3(210) Aug
set month3(240) Sep
set month3(270) Oct
set month3(300) Nov
set month3(330) Dec

if ![info exist barchart] {set barchart 0}

# x Axis by zero
#--------------------------

set p0 [nnyj 0 $j]
set item [$c create line $pxmin $p0 $pxmax $p0 -width 1.0 -fill grey -dash 3]
$c addtag axis withtag $item
$c addtag $graph withtag $item
   
# Plotting of x-axis
#--------------------------
   
set item [$c create line $pxmin $pymin $pxmax $pymin -width 1.5]
$c addtag axis withtag $item
$c addtag $graph withtag $item

# Plotting of y-axis
#--------------------------
   
set item [$c create line $pxmin $pymin $pxmin $pymax -width 1.5]
$c addtag axis withtag $item
$c addtag $graph withtag $item

# Plotting of x-axis labels
#--------------------------
  
set labelcorr 0

if { $xax == 0 } {
     set imin 0
     set anz 10
} else {
     if {$timewin >1440} {set labeldist 360}
     if {$timewin<=1440} {set labeldist 180}
     if {$timewin<=1080} {set labeldist 90}
     if {$timewin<=720} {set labeldist 30}


#      if {$timewin >1460} {set labeldist 365}
#      if {$timewin<=1440} {set labeldist 182.5}
#      if {$timewin<=1095} {set labeldist 91.5}
#      if {$timewin<=731} {set labeldist 30.5}
#     if {$timewin==1095} {set labeldist 365}
     
     set anz [expr int($xdelt/$labeldist)+1]
     set imin [expr int($xmin/$labeldist)]
     if  {[expr int(fmod($xmin,$labeldist))]!=0 } {incr imin}
}

if {$barchart==1} {set anz [expr $anz+1]}
if {$spectrum==1} {
        set imin 0
	set anz [max 1 [expr $harm*2+1]]
	if {$startharm>0} {set anz [expr $anz - 1]}
     }


 for {set i $imin} {$i <= [expr $imin+$anz]} {incr i} {

     if { $xax == 0 } {
       set xlabel [expr $xmin + $i*$xdelt/$anz]

       if {$outdelt(1) >= 1} {
         scan $xlabel %u xtext
       } else {
         set xtext [format $xaxformat $xlabel]
       }
       if {$spectrum==1} {
          set xtext [formatspec $xlabel]
	  if {$i==[expr $imin+$anz]} { set xtext " "}
	  
       }

     } else {

       set xlabel [expr $i*$labeldist]
       set xtext $xlabel

       if {$labeldist==30} {
           set xtext $month1([expr int(fmod($xlabel,360))])
           if {$timewin <= 500} {set xtext $month3([expr int(fmod($xlabel,360))])}
           set labelcorr [expr $labeldist/2.0]
       }
     }
    set specialt 0
#    if {$xgrid==1 && $cycle(1)>0 && $timewin>$cycle(1)} {specialticks $c; set specialt 1}


    if {$xlabel <= [expr $xmin+$xdelt] && $anz <= 22 } {
     set xoff [nnx $xlabel]
     set xofft [nnx [expr $xlabel+$labelcorr]]

     set item [$c create line $xoff $pymin $xoff [expr $pymin+5] -width 2]
     $c addtag axis withtag $item
     $c addtag $graph withtag $item

     set item [$c create text $xofft [expr $pymin+10] -text $xtext -anchor n -font $plotFont]
     $c addtag axis withtag $item
     $c addtag $graph withtag $item
     
     if {$xgrid==1} {
          set item [$c create line  $xoff $pymin $xoff $pymax -width 1.0 -fill grey -dash 3]
          $c addtag axis withtag $item
          $c addtag $graph withtag $item      
	 }	 
     
     
    }
}

  # Plotting of y-axis labels
  #--------------------------

for {set i 0} {$i <= 5} {incr i} {
    set ylabel [expr $ymin($j) + ($i*$ydelt/5)]
    set ytext [format "%10.3g" $ylabel]
    if {$logflag==1} {set ytext [string trim [format "%10.3g" [expr pow(10,$ylabel)]]]}

    set yoff [nnyj $ylabel $j]

    set item [$c create line [expr $pxmin-5] $yoff $pxmin $yoff -width 2]
    $c addtag axis withtag $item
    $c addtag $graph withtag $item
    set item [$c create text [expr $pxmin-10] $yoff -text $ytext -anchor e -font $plotFont]

    $c addtag axis withtag $item
    $c addtag $graph withtag $item

    if {$ygrid==1} {
          set item [$c create line  $pxmin $yoff $pxmax $yoff -width 1.0 -fill grey -dash 3]
          $c addtag axis withtag $item
          $c addtag $graph withtag $item      
    }


}

  # Bindings for horizontal line
  #-----------------------------

if {$j==0} {
  bind $c <Button-1> "xlineDown $c %x %y"
  bind $c <B1-Motion> "xlineMotion $c %x %y"
  bind $c <ButtonRelease-1> "xlineRelease $c"

} else {
  bind $c <Button-1> ""
  bind $c <B1-Motion> ""
  bind $c <ButtonRelease-1> ""
}

  # Bindings for vertical line
  #-----------------------------

if {$spectrum==0} {
    if {$j<3} {
      bind $c <Button-3> "ylineDown $c %x %y"
      bind $c <B3-Motion> "ylineMotion $c %x %y"
      bind $c <ButtonRelease-3> "ylineRelease $c"

    } else {
      bind $c <Button-3> ""
      bind $c <B3-Motion> ""
      bind $c <ButtonRelease-3> ""
    }
} else {
    if {$j<3} {
      bind $c <Button-3> "ylineDown_spec $c %x %y"
      bind $c <B3-Motion> "ylineMotion_spec $c %x %y"
      bind $c <ButtonRelease-3> "ylineRelease $c"

    } else {
      bind $c <Button-3> ""
      bind $c <B3-Motion> ""
      bind $c <ButtonRelease-3> ""
    }
 
}

}



