# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc printout {{what tool} {framecount -99}} {
global pwd DOS
global toolflag
global matlab_flag matlab_out
global xvgr_flag xvgr_out 
global gnu_flag gnu_out
global colormode
global ttt mf
global gmfile filmdir
#puts "printout what=$what"
#return
set xvgr_flag 0
set matlab_flag 0
set gnu_flag 0

set systemTime [clock seconds]
set outputdir [clock format $systemTime -format %Y_%m_%d]_[clock format $systemTime -format %H_%M]
if ![file exists $pwd/plots] {exec \mkdir $pwd/plots}
if ![file exists $pwd/films] {exec \mkdir $pwd/films}
set filecounter [clock format $systemTime -format %Y_%m_%d]_[clock format $systemTime -format %H_%M_%S]

if {$toolflag == "film"} {
   if {$framecount==-1} {
	   set filmdir $outputdir
	   if ![file exists $pwd/films/$filmdir] {exec \mkdir $pwd/films/$filmdir}
  	   textinfo "Frames are output to $pwd/films/$filmdir"
	   return
   }
} else {
}

switch $what {
xvgr {
	set xvgr_flag 1
    set file [fileselect "Select Filename" agr $pwd/plots save "xvgr_$filecounter.agr"]
    if {$file != ""} {
        set xvgr_out [open $file w 0660]
        xv_openf
        interpret
        xv_closef
        textinfo  "xmGrace output to $file"
        
        set xvgr_flag 0
	}
}
matlab {
	set matlab_flag 1
	set m_outdir "$pwd/plots/matlab_$outputdir"
    if ![file exists $m_outdir] {exec \mkdir $m_outdir}
	set file [fileselect "Select Filename" m $m_outdir save "matlab_$outputdir.m"]
	if {$file != ""} {
		set gmfile [file rootname $file]
		set matlab_out [open $file w 0660]
		matlab_openf
		interpret
		matlab_closef $file
		textinfo  "MatLab output to $file"
		set matlab_flag 0
	}
}
gnuplot {
	set gnu_flag 1
	set g_outdir "$pwd/plots/gnuplot_$outputdir"
    if ![file exists $g_outdir] {exec \mkdir $g_outdir}
	set file [fileselect "Select Filename" bat $g_outdir save "gnuplot_$outputdir.bat"]
	if {[file extension $file]!=".bat"} {set file $file.bat}
	set gmfile [file rootname $file]
	if {$file != ""} {
		set gnu_out [open $file w 0660]
		gnu_openf
		interpret
		gnu_closef $gnu_out
		textinfo  "Gnuplot output to $file"
		if {$DOS!=2} {
			cd $g_outdir
			eval exec "gnuplot -persist $file &"
			cd $pwd      
		}
	}
}
pdf {
	set file [fileselect "Select Filename" pdf $pwd/plots save "$toolflag\_$filecounter.pdf"]
	if {$file != ""} {
		set filepdf $file
		set tempfile [file rootname $file].ps
		# Papierformat für PS, PDF ist 500
		$mf.left.plot postscript -file $tempfile -colormode color -pageheight 500 -pagewidth 500
		set command "exec /usr/bin/ps2pdf12 $tempfile $filepdf"
		catch $command result
		textinfo "PDF output to $filepdf"
		puts $result   
		catch [file delete -force $tempfile]
	}
}

ps {
	set file [fileselect "Select Filename" ps $pwd/plots save "$toolflag\_$filecounter.ps"]
	if {$file != ""} {
		set fileps $file
		$mf.left.plot postscript -file $file -colormode color -pageheight 500 -pagewidth 500
		textinfo "PostScript output to $file"

	}
}

filmpng {
	set file $pwd/plots/filmsnapshot_$outputdir.png
	image create photo img -format window -data $mf.left.plot
	img write $file -format png
	textinfo "PNG output to $file"

}

film {
	set framenumber [format "%8.8d" $framecount]
	set file $pwd/films/$filmdir/frame_$framenumber.png
#	$mf.left.plot postscript -file $file -colormode color -pageheight 500 -pagewidth 500
	image create photo img -format window -data $mf.left.plot
	img write $file -format png
}	

text -
distance {
	set file [fileselect "Select Filename" txt $pwd/plots save "$toolflag\_$filecounter.txt"]
	if {$file != ""} {
		set unit [open $file w 0660]
		set text [eval {$ttt get 1.0 end}]
		puts $unit $text
		close $unit
		textinfo "Text output to $file"

	}
}
plain {
	set file [fileselect "Select Filename" txt $pwd/plots save "$toolflag\_$filecounter.txt"]
	if {$file != ""} {
		set unit [open $file w 0660]
#        interpret
		close $unit
		textinfo "Text output to $file not yet implemented"

	}
}
default {
	textinfo "unknown option $what"
	return
	}
}
}

#nicht mehr in Verwendung. Ausgabe in PS/PDF immer farbig
proc graybox {} {
global colormode
set w .graybox

toplevel $w
wm title $w "Tiger Graphics"
wm geometry $w "200x100+400+400"

label $w.msg -justify center -text "Output Style"
pack $w.msg -side top -pady {0 5}

frame $w.buttons
pack $w.buttons -side bottom -fill x -pady 2m
button $w.buttons.grey -text Grey -command "set colormode gray;destroy $w"
button $w.buttons.color -text Color -command "set colormode color;destroy $w"
pack $w.buttons.grey $w.buttons.color -side left -expand 1
}


