# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc meanplot {c j what} {
# c: canvas
# j: graph
# what: sum or mean for title

global can
global timewin
global actstart
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global colorlist colorindex
global plotFont boldFont
global singleresult command
global graph gl
global xvgr_flag matlab_flag gnu_flag
global prettyaxis
global fixymax fixymin
global multiset 0; #nur für grace output wichtig
set multiset 1
set graph [lindex $gl $j]
set lbar [llength $command($j)]

set init 1

if ![info exist prettyaxis] {set prettyaxis 0}

if { [regexp {[^0-9\.-]} $fixymin ] == 1 }  {set fixymax ""}

if {$fixymin !=""}  {
  set ymin($j) $fixymin
} else {
  set ymin($j) [min $ymin($j) 0]
}

if { [regexp {[^0-9\.-]} $fixymax ] == 1 }  {set fixymax ""}

if {$fixymax !=""}  {
  set ymax($j) $fixymax
} else {
  if {$prettyaxis==1} {
   set ymax($j) [max [rd $ymax($j)] 0]
  } else {
   set ymax($j) [max $ymax($j) 0]
  }
}

if {$ymin($j) >= $ymax($j)} {
   set ymax($j) [expr $ymin($j)+1.0]
   if {$ymin($j) > $ymax($j)} {textinfo "ymax<ymin !!"}

}

foreach selectvar $command($j) {

if { $init == 1} {

   set init 0
   set colorindex 0
   set barnum 0

   # Setting of x-axis
   set xmin $actstart(1)
   set xmax [expr $xmin + $timewin]
   set xdelt [expr {$xmax - $xmin}]
   set ydelt [expr $ymax($j) - $ymin($j)]

 
   # Plotting of x-axis and y axis
   #--------------------------
   set item [$c create line $pxmin $pymin $pxmin $pymax -width 1.5]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item

   set item [$c create line $pxmin $pymin $pxmax $pymin -width 1.5]
   $c addtag axis withtag $item
   $c addtag $graph withtag $item


  # Calculating of normalizing factor for the plot
  #--------------------------
  set pxoff [expr $pxdelt/$lbar]
  set pxlabel [expr $pxmin + $pxoff/2]

  set px1 $pxmin
  set px2 [expr $pxmin + $pxoff]

  # Plotting of header
  #--------------------------

   scan $xmin %u xtext1
   scan $xmax %u xtext2
   set graphtitle "$what over time: $xtext1 - $xtext2"
   set item [$c create text $pxmin 10 -text $graphtitle \
             -font $plotFont -anchor w]
  $c addtag axis withtag $item
  $c addtag $graph withtag $item

  # Plotting of y-axis labels
  #--------------------------
  for {set i 0} {$i <= 6} {incr i} {
    set ylabel [expr $ymin($j) + ($i*$ydelt/6)]
    set ytext [format "%10.3g" $ylabel]
    set yoff [nnyj $ylabel $j]

    set item [$c create line [expr $pxmin-5] $yoff $pxmin $yoff -width 2]
    $c addtag axis withtag $item
    $c addtag $graph withtag $item
    set item [$c create text [expr $pxmin-10] $yoff -text $ytext \
                   -anchor e -font $plotFont]
    $c addtag axis withtag $item
   $c addtag $graph withtag $item
  }

   if {$xvgr_flag == 1 } { xv_open_graph $j 0 [expr $lbar+1] \
                                      $ymin($j) $ymax($j) $graphtitle bar
                        }


   if {$matlab_flag == 1 } {matlab_open_graph $j 0 [expr $lbar+1] \
                                      $ymin($j) $ymax($j) $graphtitle bar
                        }

#    if {$gnu_flag == 1 } {gnu_open_graph $j 0 [expr $lbar+1] \
#                                       $ymin($j) $ymax($j) $graphtitle bar
#                         }



} else {set colorindex [expr $colorindex+1]}

# Plotting of singleresult
#--------------------------
set color [lindex $colorlist $colorindex]

set item [$c create text $pxlabel [expr $pymin+10] -text $selectvar \
                       -anchor n -font $plotFont -fill $color]

$c addtag $color$graph withtag $item
$c addtag $graph withtag $item

set pxlabel [expr $pxlabel + $pxoff]

# Preparing data for plot
#------------------------
set pmean [nnyj $singleresult($selectvar) $j]

# Plotting one bar
#------------------------

set item [eval {$c create poly} \
          $px1 $pymin $px1 $pmean $px2 $pmean $px2 $pymin  \
               {-width 1 -fill $color}]
$c addtag $color$graph withtag $item
$c addtag $graph withtag $item
$c bind $color$graph <3> "set eraseset 1;makecommand -1 -1 $colorindex $j"

set px1 $px2
set px2 [expr $px2 + $pxoff]

    if {$xvgr_flag ==1 && $multiset==0 } {
       xv_set_bar $barnum $singleresult($selectvar) $selectvar
      incr barnum
    }

    if {$matlab_flag ==1} {
       matlab_add_bar $barnum $singleresult($selectvar) $selectvar
      incr barnum
    }

    if {$gnu_flag ==1} {
       gnu_add_bar $barnum $singleresult($selectvar) $selectvar
      incr barnum
    }

}

if {$matlab_flag ==1} {
    matlab_plot_bars
}

if {$gnu_flag ==1} {
    gnu_plot_bars $j $graphtitle
}


# Sonderfunktion fuer Multiset-Barcharts, darstellung in grace als Einzelwerte eines grace-sets
#------------------------------------------------------------------------------------------------------------------------------

if {$xvgr_flag ==1 && $multiset==1} {
   set index 1
   foreach selectvar $command($j) {
      xv_set_tickmark $index $selectvar
      incr index
   }
   set setindex 0
  
     xv_open_multibar_header $j $setindex ""
     set index 1
     foreach selectvar $command($j) {
       set pmean $singleresult($selectvar)
       xv_set_multibar $index $pmean
       incr index
     }
   xv_close_multibar_header
   incr setindex
   
}



}



