# MOVIE 5.0 Copyright (C) 2023, Cora Kohlmeier, Frank Hamberg

proc profcom_plot {j} {
global can mf
global startbox boxwindow
global outdelt timewin
global actstart
global pxmin pxmax pymin pymax pxdelt pydelt
global xmin xmax ymin ymax xdelt ydelt
global colorlist colorindex
global plotFont boldFont
global command
global graph gl
global vertoff
global sameaxis calcaxis calcext globalmin globalmax smooth
global filename filelist longnames
global singleresult 
global timemin timemax
global timemaxflag
global xax
global layer layersum layervalue tiefe
global xvgr_flag
global fixymax fixymin xax prettyaxis
global barchart flipxy valu
global sets

set graph [lindex $gl $j]
set selectvar [lindex $command($j) 0]

set init 1
set c $mf.left.plot  


if {$timemaxflag==1 && $sameaxis!=1} {
  set ymin($j) $timemin($j)
  set ymax($j) $timemax($j)
}

if { [regexp {[^0-9\.-]} $fixymin ] == 1 }  {set fixymax ""}

if {$fixymin !=""}  {
  set ymin($j) $fixymin
} else {
  set ymin($j) [min $ymin($j) 0]
}

if { [regexp {[^0-9\.-]} $fixymax ] == 1 }  {set fixymax ""}

if {$fixymax !=""}  {
  set ymax($j) $fixymax
} else {
  set ymax($j) [max $ymax($j) 0]
}

if {$ymin($j) >= $ymax($j)} {
   set ymax($j) [expr $ymin($j)+1.0]
   if {$ymin($j) > $ymax($j)} {textinfo "ymax<ymin !!"}

}

set init 0
   set colorindex 0
   set setnum 0

   if {$barchart==1} {
     set xmin $startbox
     set xmax [expr $startbox + $boxwindow+1]
     set xdelt [expr $boxwindow+1]
   } else {
     set xmin 0
     set xmax $layersum
     set xdelt $layersum
   }


   set ydelt [expr $ymax($j) - $ymin($j)]

# Plotting of axes
#--------------------------

   if {$flipxy==0} {paxes $j $c} else {flippaxes $j $c}

   if {$xvgr_flag == 1} { xv_open_graph $j \
                                     $ymin($j) \
                                     $ymax($j) \
                                     $xmin $xmax " " xx $flipxy
                        }

# Plotting of variable name
#--------------------------
set header [makeheader $selectvar]
set item [$c create text 100 30 -text $header \
          -font $plotFont -anchor w]
$c addtag axis withtag $item
$c addtag $graph withtag $item


set sets [llength $filelist]

foreach k $filelist {

  set xv_x($k) ""
  set xv_y($k) ""
   # Plotting of legend
   #-------------------

      if {$longnames==0} {
        set legendtext($k) [file rootname [file tail $filename($k)]]
      } else {
        set legendtext($k)  $filename($k)
      }
   set color [lindex $colorlist $colorindex]
   set row [expr 20+$vertoff*$colorindex]

   if {$j == 0} {


    set item [$c create text 450 $row -text $legendtext($k) \
             -font $plotFont -fill $color -anchor w]
    $c addtag $color$graph withtag $item
    $c addtag legend$graph withtag $item
    $c addtag $graph withtag $item
 
    $c bind legend$graph <1> "complegendDown $c %x %y legend$graph;\
                           $c itemconfig legend$graph -font {$boldFont}"
    $c bind legend$graph <ButtonRelease-1> "complegendMove $c %x %y;\
                           $c dtag selected;\
                           $c itemconfig legend$graph -font {$plotFont}"


    set compleg(lastX) 0
    set compleg(lastY) 0
   }

  # Plotting of data
  #-----------------
  if [info exist plotdata] {unset plotdata}
  if [info exist posdata] {unset posdata}

# Plotting one data set
#------------------------

if {$barchart==0} {
# GRAPH PLOT
#-----------
   set pos $xmax
   set i $startbox

   foreach y $singleresult($k,$selectvar) {
   if {$y==""} {set y 0}


      if {$flipxy==0} {
         lappend plotdata [nnx [expr $xmax-$pos]] [nnyj $y $j]
         set pos [expr $pos-$layer($i)]
         if {$layervalue==1} {
            lappend xv_x($k)  [string trim [format "%10.3g" $tiefe($i)]]
          } else {
            lappend xv_x($k) $i
          }
         lappend xv_y($k) $y

	

      } else {
         lappend plotdata [nx $y $ymin($j) $ymax($j)] [ny $pos $xmin $xmax]
         set pos [expr $pos-$layer($i)]

         lappend xv_x($k) $y
         if {$layervalue==1} {
           lappend xv_y($k)  [string trim [format "%10.3g" $tiefe($i)]]
         } else {
           lappend xv_y($k) $i
         }

       
     }
       
     
     lappend posdata $pos
     incr i
   }


    if {$smooth ==1} {set sm true} else {set sm false}

    set item [eval {$c create line} $plotdata {-width 1 -fill $color -smooth $sm }]


    $c addtag $color$graph withtag $item
    $c addtag $graph withtag $item
    $c bind $color$graph <3> "set eraseset 1;makecommand -1 -1 $colorindex $j"

} else {

# BARCHARTS
#----------

if {$flipxy==0} {
    set pos [expr (1.0*$setnum)/$sets+1]
 } else {
    set pos [expr $xmax-(1.0*$setnum)/$sets]
 }

 foreach y $singleresult($k,$selectvar) {

    set offset [expr 1.0/$sets]
    if {$colorindex==0} {set fillcolor grey} else {set fillcolor $color}

    if {$flipxy==1} {
      set item [eval {$c create rect} [nx 0 $ymin($j) $ymax($j)] [ny $pos $xmin $xmax] \
                                      [nx $y $ymin($j) $ymax($j)] [ny [expr $pos-$offset] $xmin $xmax]\
                                      {-width 1 -fill $fillcolor}]
    } else {
      set item [eval {$c create rect} [nnx $pos] [nnyj 0 $j] \
                                      [nnx [expr $pos+$offset]] [nnyj $y $j]\
                                      {-width 1 -fill $fillcolor}]
    }

    if {$flipxy==0} {
      set pos [expr $pos+1]
    } else {
      set pos [expr $pos-1]
    }


    $c addtag $color$graph withtag $item
    $c addtag $graph withtag $item
    $c bind $color$graph <3> "set eraseset 1;makecommand -1 -1 $colorindex $j"

  }
}


if {$xvgr_flag ==1 } {    
    xv_set_num $j [expr $k-1] $xv_y($k) "$header $legendtext($k)" 1 $xv_x($k)    
}

  incr setnum
  set colorindex [expr $colorindex+1]

}
if {$valu==1} {profvaluplot $j $row [llength $filelist]}
}


